'*********************************************************************************
'*  Name    : SHT demo program                                                   *
'*  File    : SHT_4Lines_18F252.bas                                              *
'*  Author  : [select VIEW...EDITOR OPTIONS]                                     *
'*  Notice  : Copyright (c) 2005 [select VIEW...EDITOR OPTIONS]                  *
'*          : All Rights Reserved                                                *
'*  Date    : 30.12.2005                                                         *
'*  Version : 1.0                                                                *
'*  Notes   : Based on the sample code from Brandon Tarr?? (proton+ user website)*
'*          : Display alpha type 4 x 20                                          * 
'*********************************************************************************
'
    Device	18F252
    Xtal 4
'
'    BOOTLOADER On 
    ADCON1 = 7 
    TRISB=%11000000                     ' Setup PORTB
'       
'++++   Declares for LCD   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    Declare LCD_Type 0                  ' Alpha-Type
    Declare LCD_DTPin PORTB.0           ' 4 lower Pins are used      
    Declare LCD_ENPin PORTB.4
    Declare LCD_RSPin PORTB.5 
    Declare LCD_Interface 4
    Declare LCD_Lines 4 
'
'++++   Declares for SHT       +++++++++++++++++++++++++++++++++++++++++++++++++++++
    Symbol ShtStatus_W  = %00110        ' SHT status register write
    Symbol ShtStatus_R  = %00111        ' SHT status register read
    Symbol ShtMeas_Temp = %00011        ' read SHT temperature  
    Symbol ShtMeas_Humi = %00101        ' read SHT humidity           
    Symbol ShtReset     = %11110        ' soft reset 
'               
'++++   Declares for SHT Port  +++++++++++++++++++++++++++++++++++++++++++++++++++++    
    Symbol ShtData   = PORTC.1          ' SHT data line
    Symbol ShtClock  = PORTC.0          ' SHT SCK line
'
    Symbol Ack      = 0
    Symbol NoAck    = 1
    Symbol No       = 0
    Symbol Yes      = 1
    Symbol DegSym   = 223               ' degrees symbol for DEBUG
'    
    Dim ioByte	    As Byte             ' data from/to SHT1x
    Dim ackBit	    As Bit              ' ack/nak from/to SHT1x
    Dim toDelay     As Byte             ' timeout delay timer
    Dim timeOut     As Bit              ' timeout status
    Dim soT		    As Word             ' temp counts from SHT1x
    Dim soTf	    As Word
    Dim tC		    As Float            ' temp - celcius
    Dim tF		    As Float            ' temp - fahrenheit
    Dim soRH	    As Word             ' humidity counts from SHT1x
    Dim soRHf	    As Float
    Dim soRH2	    As Float            ' soRHf * soRHf
    Dim rhCoeff	    As Float
    Dim rhLin	    As Float            ' humidity; linearized
    Dim rhTrue	    As Float            ' humidity; temp compensated
    Dim statu	    As Byte             ' SHT1x status byte
    Dim DewPoint    As Float
    Dim TempDP_1     As Float
    Dim TempDP_2     As Float    
    Dim TempDP_3     As Float    
' Initialization
' ------------------------------------------------------------------------------
    Initialize:
    GoSub SHT_Connection_Reset              ' reset device connection
    
' Program Code
' ------------------------------------------------------------------------------
Main:
    Cls
    DelayMS 20
    Print At 1,1,"   SHT1x Demo   "
    Print At 2,1,"   ----------   "
    DelayMS 1000
    Cls
    DelayMS 20
'    
Main_1:
    Cls
    GoSub SHT_Measure_Temp
    Print At 1,1, "SHT_Measure_Temp"
    Print At 2,1, "soT =    ", Dec soT,"     "
    Print At 3,1, "Temp C:  ", Dec2 tC," ", DegSym, "C  "  
    Print At 4,1, "Temp F:  ", Dec2 tF," ", " F  " 
    DelayMS 5000
    Cls
    DelayMS 20
'    
    GoSub SHT_Measure_Humidity
    Print At 1,1, "SHT_Measure_Humidity"
    Print At 2,1, "soRH:   ", Dec soRH
    Print At 3,1, "rhLin:  ", Dec2 rhLin, " %L "
    Print At 4,1, "rhTrue: ", Dec2 rhTrue, " %T"
'    
    DelayMS 5000                            ' minimum delay between readings
'    
    Cls
    Print At 1,1, "SHT_DEWPOINT"
    Print At 2,1, "DewPoint: ", Dec2 DewPoint, " ", DegSym, "C  " 
'    
    DelayMS 5000                            ' minimum delay between readings
'
    GoTo Main_1
    End
'**********************************************************************************    
' Subroutines
'**********************************************************************************
'
' ---------------------------------------------------------------------------------
SHT_Connection_Reset:
' connection reset: 9 ShtClock cyles with ShtData high, then start sequence
'      _____________________________________________________         ________
'DATA:                                                      |_______|
'         _    _    _    _    _    _    _    _    _        ___     ___
'SCK : __| |__| |__| |__| |__| |__| |__| |__| |__| |______|   |___|   |______
'
    SHOut ShtData, ShtClock, lsbfirst, [$FFF\9]
'
' ---------------------------------------------------------------------------------    
SHT_Start:    
' generates a transmission start 
'          _____         _______
' ShtData       |_______|
'              ___     ___
' ShtClock ___|   |___|   |_____
'
    High ShtData: Low ShtClock              ' Initial state
    DelayMS 1
    High ShtClock
        DelayMS 5 
    Low ShtData
        DelayMS 5
    Low ShtClock 
        DelayMS 15         
    High ShtClock 
        DelayMS 5         
    High ShtData
        DelayMS 5
    Low ShtClock
        DelayMS 5
    Return
'
' ---------------------------------------------------------------------------------    
' measure temperature
' -- celcius = soT * 0.01 - 40
' -- fahrenheit = soT * 0.018 - 40
'
SHT_Measure_Temp:
    GoSub SHT_Connection_Reset

    GoSub SHT_Start                         ' alert device
    ioByte = ShtMeas_Temp                   ' temperature command

    GoSub SHT_Write_Byte                    ' send command

    GoSub SHT_Wait_Temp                     ' wait until measurement done
    ackBit = Ack                            ' another read follows

    GoSub SHT_Read_Byte                     ' get MSB
    soT.HighByte = ioByte
    ackBit = NoAck                          ' last read

    GoSub SHT_Read_Byte                     ' get LSB
    soT.LowByte = ioByte   
    soTf = soT    
    tC = (0.01 * soTf)
    tC = tC - 40 
    tF = 1.8 * tC + 32           
    Return
'
' ---------------------------------------------------------------------------------     
' measure humidity
'
SHT_Measure_Humidity:
    GoSub SHT_Connection_Reset

    GoSub SHT_Start                         ' alert device
    ioByte = ShtMeas_Humi                   ' humidity command

    GoSub SHT_Write_Byte                    ' send command

    GoSub SHT_Wait_Hum                      ' wait until measurement done
    ackBit = Ack                            ' another read follows

    GoSub SHT_Read_Byte                     ' get MSB
    soRH.HighByte = ioByte
    ackBit = NoAck                          ' last read

    GoSub SHT_Read_Byte                     ' get LSB
    soRH.LowByte = ioByte
'    
' linearize humidity
' rhLin = (soRH * 0.0405) - (soRH^2 * 0.0000028) - 4
    soRHf = soRH
    soRH2 = soRHf * soRHf
    rhLin = 0.0405 * soRH
    rhLin = rhLin - (0.0000028 * soRH2)
    rhLin = rhLin - 4
'
' rh_true=(t_C-25)*(T1+T2*rh)+rh_lin;   
' temperature compensated humidity
    rhCoeff = (0.01 + 0.00008 * soRHf)
    rhTrue = tC - 25
    rhTrue = (rhTrue * rhCoeff) + rhLin   
'
' calculate dewpoint
    TempDP_1 = Log10 rhTrue   
    TempDP_1 = TempDP_1 -2 
    TempDP_1 = TempDP_1 / 0.4343
    
    TempDP_2 = (17.62 * tC)
    TempDP_3 = 243.12 + tC
    TempDP_3 = TempDP_2 / TempDP_3
    TempDP_3 = TempDP_1 + TempDP_3
    TempDP_1 = 243.12 * TempDP_3
    TempDP_2 = 17.62 - TempDP_3
    DewPoint = TempDP_1 / TempDP_2  
    Return
' ---------------------------------------------------------------------------------     
' sends "status"
'
SHT_Write_Status:
    GoSub SHT_Start                         ' alert device
    ioByte = ShtStatus_W                    ' write to status reg command
    GoSub SHT_Write_Byte                    ' send command
    ioByte = statu
    GoSub SHT_Write_Byte
    Return
'
' ---------------------------------------------------------------------------------     
' returns "status"
'
SHT_Read_Status:
    GoSub SHT_Start                         ' alert device
    ioByte = ShtStatus_W                    ' write to status reg command
    GoSub SHT_Read_Byte                     ' send command
    ackBit = NoAck                          ' only one byte to read
    GoSub SHT_Read_Byte
    Return
'
' ---------------------------------------------------------------------------------     
' sends "ioByte"
' returns "ackBit"
'
SHT_Write_Byte:
    SHOut ShtData, ShtClock, msbfirst, [ioByte] ' send byte
    SHIn ShtData, ShtClock, lsbpre, [ackBit\1]  ' get ack bit
    Return
'
' ---------------------------------------------------------------------------------     
' returns "ioByte"
' sends "ackBit"
'
SHT_Read_Byte:
    SHIn ShtData, ShtClock, msbpre, [ioByte]        ' get byte
    SHOut ShtData, ShtClock, lsbfirst, [ackBit\1]   ' send ack bit

    Low ShtData                                 ' release data line
    Low ShtData                                 ' release data line    
    Return
'
' ---------------------------------------------------------------------------------     
' wait for device to finish measurement
'
SHT_Wait_Temp:
    DelayMS 250
    Return
'
' --------------------------------------------------------------------------------- 
'    
SHT_Wait_Hum:
    DelayMS 70
    Return
'
' ---------------------------------------------------------------------------------      
' reset SHT1x with soft reset
'
SHT_Soft_Reset:
    GoSub SHT_Connection_Reset                  ' reset the connection
    ioByte = ShtReset                           ' reset command
    ackBit = NoAck                              ' only one byte to send
    GoSub SHT_Write_Byte                        ' send it
    DelayMS 11                                  ' wait at least 11 ms
    Return
    
    End
' ---------------------------------------------------------------------------------     
